/*
This file is part of Trabucco.

Trabucco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Trabucco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Trabucco.  If not, see <http://www.gnu.org/licenses/>.

Copyright (C) 2016  Salvo "LtWorf" Tomaselli
Copyright (C) 2016 Giuseppe Bilotta
*/

#ifndef DOWNLOADER_H
#define DOWNLOADER_H

#include <QObject>
#include <QStringList>
#include <QString>
#include <QFile>
#include <QNetworkAccessManager>
#include <QNetworkReply>

class Downloader : public QObject {
    Q_OBJECT
public:
    explicit Downloader(QStringList sources, QString destination, QObject *parent = 0);
private slots:
    void download();
    void finished(QNetworkReply* reply);
private:
    QStringList sources;
    QString destination;
    QFile dest_file;
    QNetworkAccessManager nam;

};

#endif // DOWNLOADER_H
